package uprrp.tania.networking;

import android.os.AsyncTask;
import android.util.Log;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.net.URL;
import java.net.URLEncoder;

import javax.net.ssl.HttpsURLConnection;

import uprrp.tania.utils.URLEventListener;

public class SendLoginCredentials extends AsyncTask<String, Void, String> {

    private static final String TAG = "SendAccountRecovery";
    private static final String loginBaseURL = "https://tania.uprrp.edu/recoverAccount.php";
    private final URLEventListener myCallback;

    public SendLoginCredentials(URLEventListener callback) {
        this.myCallback = callback;
    }

    private boolean validInputs(String... strings) {

        if(strings.length != 3) {
            Log.e(TAG, "Invalid string array length!");
            return false;
        }

        for(int i = 0; i < strings.length; i++) {
            if(strings[i] == null) {
                Log.e(TAG, "Encountered null parameter on index " + i);
                return false;
            }
        }

        return true;

    }

    // TODO: instead of making a normal POST request with
    //  the body data=<JSON>, we should just send the JSON itself
    //  This is adding unnecessary layers of complexity when decoding AND encoding
    /*
        JSON format:
        {
            "email": String,
            "password": String,
            "token": String,
        }
    */
    @Override
    protected String doInBackground(String... strings) {

        // Validation
        if(!validInputs(strings)) {
            Log.e(TAG, "Invalid inputs given!");
            return null;
        }

        // Extract variables
        String deviceToken = strings[0];
        String email = strings[1];
        String password = strings[2];

        try {

            // Create JSON
            JSONObject accountRecoveryJSON = new JSONObject();
            accountRecoveryJSON.put("email", email);
            accountRecoveryJSON.put("password", password);
            accountRecoveryJSON.put("token", deviceToken);
            Log.d(TAG, "RecoveryJSON: " + accountRecoveryJSON.toString());

            // Encode data
            String encodedRequestBody = URLEncoder.encode("data", "UTF-8") + "=" + URLEncoder.encode(accountRecoveryJSON.toString(), "UTF-8");

            // Send POST data request
            URL url = new URL(loginBaseURL);
            HttpsURLConnection conn = (HttpsURLConnection) url.openConnection();
            conn.setDoOutput(true);
            OutputStreamWriter wr = new OutputStreamWriter(conn.getOutputStream());
            wr.write(encodedRequestBody);
            wr.flush();

            // Get the server response
            BufferedReader serverReader = new BufferedReader(new InputStreamReader(conn.getInputStream()));
            StringBuilder serverResponse = new StringBuilder();

            String line = "";
            while(line != null) {
                serverResponse.append(line);
                line = serverReader.readLine();
            }

            String response = serverResponse.toString();
            Log.d(TAG, "The server's response is: " + response);
            if(response.contains("Error")) {
                return null;
            } else {
                return response;
            }

        } catch (JSONException e) {
            Log.e(TAG, "Couldn't prepare accountRecoveryJSON!");
            e.printStackTrace();
            return null;
        } catch (UnsupportedEncodingException e) {
            Log.e(TAG, "Couldn't encode accountRecoveryJSON!");
            e.printStackTrace();
            return null;
        } catch(Exception e) {
            Log.e(TAG, "Couldn't communicate with server while recovering account!");
            e.printStackTrace();
            return null;
        }

    }

    @Override
    protected void onPostExecute(String response) {

        if(this.myCallback == null) {
            Log.e(TAG, "Callback wasn't initialized first!");
            return;
        }

        if(response == null) {
            this.myCallback.onFailure(new Exception("Error occurred during transaction!"));
            return;
        }

        this.myCallback.onSuccess(response);

    }

}
